@@ -118,7 +118,7 @@ class LinuxDevice(object):
     '''
     A base class for common linux devices.
     '''
-    release = '1.1.33'
+    release = '1.1.35'
 
     LOG_EMERG = 0   # not used
     LOG_ALERT = 1   # not used
@@ -520,14 +520,19 @@ class LinuxDevice(object):
         now = time.strftime('%Y%m%d %H:%M:%S',time.localtime(time.time()))
         l = self['log_time'] and "[%s]" %(now) or ""
         #l += "[%s][%s]" %(now, threading.current_thread().name)
-        l += "[%s%s%s]" %(name_color and name_color or "",
+        l2 = l + "[%s%s%s]" %(name_color and name_color or "",
                 self["name"] and self["name"] or self.name,
                 name_color and "\033[0m" or "")
+        l3 = l + "[%s]" %(self["name"] and self["name"] or self.name)
 
         f = StringIO(message)
         for i in f.readlines():
             # remove the trailing '\n' and readd later.
             i = i.strip('[\r\n]')
+            if self.fd:
+                self.fd.write("%s: %s\n" %(l3, i))
+                self.fd.flush()
+
             if color != "no_color" or bg_color != "no_color":
                 i = "%s%s%s%s" %(color_dict.get(color, 'no_color'),
                         bg_color_dict.get(bg_color, 'no_color'),
@@ -535,10 +540,7 @@ class LinuxDevice(object):
                         color_dict['no_color'])
             if leading:
                 i = leading + i
-            sys.stdout.write("%s: %s\n" %(l, i))
-            if self.fd:
-                self.fd.write("%s: %s\n" %(l, i))
-                self.fd.flush()
+            sys.stdout.write("%s: %s\n" %(l2, i))
         f.close()
 
     def sleep (self, timeout, total = 50, char = '>', description = "sleep"):
