@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-import sys, os, time, socket, re
+import sys, os, time, socket, re, copy
 from netdevice import linux
 import simplejson as json
 import ipaddress
@@ -20,34 +20,48 @@ except ImportError:
     import urllib.parse as urlparse
 
 ovstools_script = '''#!/bin/bash
+#!/bin/bash
 #Transfer the ip address from $bridge to $physicial interface.
 # INPUT:
 #     br: bridge name
 #     ifname: interface name
 # OUTPUT:
 #     null
+# sh .netdevice_ovstools.sh takeover eno1 br0 eno1
 function ovsif_takeover()
 {
-    local br=$1
-    local ifname=$2
+    local ifname=$1
+    local br=$2
+    local port=$3
     local ip4=""
     local ip6=""
-    local mac=""
+    #local mac=""
     local route=""
 
     ip4=$(ip -o -4 addr list $ifname | awk '{print $4}')
     # TODO:IPV6
     ip6=$(ip -o -6 addr list $ifname | awk '{print $4}')
-    mac=$(ip link show $ifname | grep "link/ether" | awk '{print $2}')
-    IFS=$'\n'
+    #mac=$(ip link show $ifname | grep "link/ether" | awk '{print $2}')
+    IFS=$'
+'
     route=($(ip route show | grep $ifname | grep -v link | sed "s/$ifname/$br/g"))
 
-    echo "Take over $ip4 from $ifname to $br."
+    echo "#Take over $ip4 from $ifname to $br($port)."
     ip addr flush dev $ifname
-    ovs-vsctl add-port $br $ifname
-    ovs-vsctl set bridge $br other_config:hwaddr=$mac
+    echo "ip addr flush dev $ifname"
+
+    if [[ $ifname == $port ]]
+    then
+        # Add the physicial interface to the bridge $br
+        ovs-vsctl add-port $br $ifname
+        echo "ovs-vsctl add-port $br $ifname"
+    fi
+
+    #ovs-vsctl set bridge $br other_config:hwaddr=$mac
     ip link set $br up
+    echo "ip link set $br up"
     ip address add $ip4 dev $br
+    echo "ip address add $ip4 dev $br"
 
     # transfer the route from $ifname to $br
     for line in ${route[@]};
@@ -67,30 +81,35 @@ function ovsif_takeover()
 #     null
 function ovsif_restore()
 {
-    local br=$1
-    local ifname=$2
+    local ifname=$1
+    local br=$2
+    local port=$3
     local ip4=""
     local ip6=""
     local route=""
 
     ip4=$(ip -o -4 addr list $br | awk '{print $4}')
     # TODO:IPV6
-    ip6=$(ip -o -6 addr list $ifname | awk '{print $4}')
-    IFS=$'\n'
+    ip6=$(ip -o -6 addr list $br | awk '{print $4}')
+    IFS=$'
+'
     route=($(ip route show | grep $br | grep -v link | sed "s/$br/$ifname/g"))
 
-    echo "restore $ip4 from $br to $ifname."
+    echo "#restore $ip4 from $br to $ifname, del $port from $br."
     ip addr flush dev $br
-    ovs-vsctl del-port $br $ifname
+    echo "ip addr flush dev $br"
+    ovs-vsctl del-port $br $port
+    echo "ovs-vsctl del-port $br $port"
     ip address add $ip4 dev $ifname
-
+    echo "ip address add $ip4 dev $ifname"
+    
     # transfer the route from $ifname to $br
     for line in ${route[@]};
-    do
+    do  
         #echo "ip route add $line"
         eval ip route add $line
     done
-
+    
     return 0
 }
 
@@ -115,6 +134,14 @@ function main()
         return 0
     fi
 
+    local pid_num_of_ovs=0
+    pid_num_of_ovs=$(ps -ef | grep -w ovs-vswitchd | grep -v grep | wc -l)
+    if test $pid_num_of_ovs -le 0
+    then
+        echo "It seems that OVS is not running. Please start it at first."
+        return 0
+    fi
+
     case "$1" in
         takeover)
             shift
@@ -238,11 +265,7 @@ class OvsHost(linux.LinuxDevice):
         ports = ports if (isinstance(ports, list)) else [ports]
         ports = ports if (not args) else ports.extend(args)
         for p in ports:
-            if (p.get("type", "normal") == "normal") and (p["name"] in self.nics):
-                #If it's a normal interface, take over it
-                self.takeover_interface(name, p["name"])
-            else:
-                self.add_port(name, p)
+            p["mac"] = self.add_port(name, **p)
         if bonds:
             self.add_bond(name, bonds["name"], *bonds["port"])
 
@@ -258,17 +281,30 @@ class OvsHost(linux.LinuxDevice):
         '''
         Add bond.
         '''
+        ip = kwargs.pop("ip", []) #ip is private paramater.
         for p in args:
             self.cmd("ip addr flush dev %s" %(p))
-        cmd = "ovs-vsctl add-bond %s %s " %(bridge_name, bond_name)
-        cmd += " ".join(args)
+        cmd = "ovs-vsctl add-bond %s %s" %(bridge_name, bond_name)
+        cmd += " " + " ".join(args)
+        cmd += " " + " ".join('%s=%s' %(k,v) for k,v in kwargs.items())
         self.cmd(cmd)
 
-    def add_port (self, bridge_name, kwargs):
+        if ip:
+            # Configure the ip address for the address for route
+            self.cmd('ip link set %s up' %(bridge_name))
+            self.cmd('ip address add %s dev %s' %(ip, bridge_name))
+
+    def add_port (self, bridge_name, name, **kwargs):
         '''
-        Add self-port and do some configuration if necessary.
+        Add a port to the brige and do some configuration if necessary.
+        return the mac address of the port if has.
         '''
-        name = kwargs.get("name", "tmpname")
+        if (kwargs.get("type", "normal") == "normal") and \
+                (name in self.nics):
+            #If it's a normal interface, take over it
+            self.takeover_interface(bridge_name, name, name)
+            return None
+
         command = 'ovs-vsctl --may-exist add-port %s %s' %(bridge_name, name)
 
         # parameter ip/mac is not valid, won't be included in the add-port
@@ -282,14 +318,8 @@ class OvsHost(linux.LinuxDevice):
                 command += ' %s=%s' %(k,v)
         self.cmd(command) #execut the command.
 
-        #If it's a normal interface, flush it to release its original ip address.
-        #if (kwargs.get("type", "normal") == "normal"):
-        #    #self.cmd("ip addr flush dev %s" %(name))
-        #    #self.cmd("sh ovstools.sh takeover %s %s" %(bridge_name, name))
-        #    self.takeover_interface(bridge_name, name)
-
         macaddr = self.ovs_get_interface(name, "mac_in_use").strip('"')
-        kwargs["mac"] = macaddr
+        #kwargs["mac"] = macaddr
         self.macs.append(macaddr)
         if kwargs.get("ip", None) :
             # Configure the ip address for the address for route
@@ -304,10 +334,11 @@ class OvsHost(linux.LinuxDevice):
                     'priority=10000,arp,arp_tpa=%s,arp_op=1,actions=move:"NXM_OF_ETH_SRC[]->NXM_OF_ETH_DST[]",mod_dl_src:"%s",load:"0x02->NXM_OF_ARP_OP[]",move:"NXM_NX_ARP_SHA[]->NXM_NX_ARP_THA[]",load:"0x%s->NXM_NX_ARP_SHA[]",move:"NXM_OF_ARP_SPA[]->NXM_OF_ARP_TPA[]",load:"0x%s->NXM_OF_ARP_SPA[]",in_port'%(ipaddr, macaddr, re.sub(r":", "", macaddr), "".join(f"{i:02x}" for i in ipaddr.packed)),
                     'priority=10000,icmp,nw_dst=%s,icmp_type=8,icmp_code=0,actions=push:"NXM_OF_ETH_SRC[]",push:"NXM_OF_ETH_DST[]",pop:"NXM_OF_ETH_SRC[]",pop:"NXM_OF_ETH_DST[]",push:"NXM_OF_IP_SRC[]",push:"NXM_OF_IP_DST[]",pop:"NXM_OF_IP_SRC[]",pop:"NXM_OF_IP_DST[]",load:"0xff->NXM_NX_IP_TTL[]",load:"0->NXM_OF_ICMP_TYPE[]",in_port' %(ipaddr),
                     )
+        return macaddr
 
-    def takeover_interface (self, bridge_name, ifname):
+    def takeover_interface (self, bridge_name, ifname, portname = None):
         '''
-        takeover the @ifname to @bridge_name
+        takeover the ip from @ifname to @bridge_name(@portname)
         '''
         filename = '.netdevice_ovstools.sh'
         if self.cmd("[ -f ~/%s ] || echo no" %(filename)).strip() == "no":
@@ -315,19 +346,21 @@ class OvsHost(linux.LinuxDevice):
                 f.write(ovstools_script)
             self.put_file(filename, "~/")
 
-        self.cmd("sh ~/%s takeover %s %s" %(filename, bridge_name, ifname))
+        self.cmd("sh ~/%s takeover %s %s %s" %(filename,
+            ifname, bridge_name, portname and portname or ifname))
 
-    def no_takeover_interface (self, bridge_name, ifname):
+    def no_takeover_interface (self, bridge_name, ifname, portname = None):
         '''
-        takeover the @ifname to @bridge_name
+        restore the ip from @bridge_name(@portname) to @ifname
         '''
-        filename = '.netdevice_ovstools.sh'
+        filename = ".netdevice_ovstools.sh"
         if self.cmd("[ -f ~/%s ] || echo no" %(filename)).strip() == "no":
             with open(filename, 'w') as f:
                 f.write(ovstools_script)
             self.put_file(filename, "~/")
 
-        self.cmd("sh ~/%s restore %s %s" %(filename, bridge_name, ifname))
+        self.cmd("sh ~/%s restore %s %s %s" %(filename,
+            ifname, bridge_name, portname and portname or ifname))
 
     def add_device (self, bridge_name, *args, **kwargs):
         '''
@@ -436,8 +469,9 @@ class OvsHost(linux.LinuxDevice):
 
         # 1) Add vhostuser port
         for p in vm.get("port", []):
-            device = self._util_dictionary_parse(p["-device"].split(","))
-            if "vhost-user" in p["-netdev"]:
+            #self.log("__add_vm(), p: %s" %(p))
+            #device = self._util_dictionary_parse(p["-device"].split(","))
+            if p.get("-netdev", None) and "vhost-user" in p["-netdev"]:
                 chardev = self._util_dictionary_parse(p["-chardev"].split(","))
                 if ("server" in chardev):
                     # vhostuserclient
@@ -445,12 +479,11 @@ class OvsHost(linux.LinuxDevice):
                     ovs_port = {"name": port_name,
                             "type": "dpdkvhostuserclient",
                             "options:vhost-server-path": chardev["path"]}
-                    self.add_port(bridge_name, ovs_port)
                 else:
                     # type: vhostuser
                     port_name = os.path.basename(chardev["path"])
                     ovs_port = {"name": port_name, "type": "dpdkvhostuser"}
-                    self.add_port(bridge_name, ovs_port)
+                ovs_port["mac"] = self.add_port(bridge_name, **ovs_port)
 
         # 2)start the qemu: tap mode need to start the qemu at frist,
         # vhostuser need to start the ovs-port at first. 
@@ -459,8 +492,14 @@ class OvsHost(linux.LinuxDevice):
 
         # 3)add the tap port, and add the flow
         for p in vm.get("port", []):
+            #self.log("__add_vm(), p: %s" %(p))
             device = self._util_dictionary_parse(p["-device"].split(","))
-            netdev = self._util_dictionary_parse(p["-netdev"].split(","))
+            if (device["type"] == "virtio-net-pci" and p.get("-netdev", None)):
+                netdev = self._util_dictionary_parse(p["-netdev"].split(","))
+            else:
+                self.log("OVS doesn't support device: %s." %(device["type"]))
+                continue
+
             if "vhost-user" in p["-netdev"]:
                 br = bridge_name
                 pass
@@ -473,7 +512,7 @@ class OvsHost(linux.LinuxDevice):
                 if (netdev.get("script", "/etc/qemu-ifup") == "no"):
                     # There is no user-defined script, we add it by ourselves:
                     self.cmd("ip link set %s up" %(port_name))
-                    self.add_port(br, ovs_port)
+                    ovs_port["mac"] = self.add_port(br, **ovs_port)
             elif "bridge" in p["-netdev"]:
                 # type: tap
                 #netdev = self._util_dictionary_parse(p["-netdev"].split(","))
@@ -482,7 +521,7 @@ class OvsHost(linux.LinuxDevice):
                 br = netdev.get("br", "br0") #default=br0
                 # qemu would add the port automatically for tap interface.
                 self.cmd("ip link set %s up" %(port_name))
-                self.add_port(br, ovs_port) #default=br0
+                ovs_port["mac"] = self.add_port(br, **ovs_port) #default=br0
             else:
                 # Now only vhost-user and tap will be added into OVS
                 continue
@@ -548,7 +587,8 @@ class OvsHost(linux.LinuxDevice):
 
         for d in args:
             #self.log("d: %s" %(d))
-            if (d["mac"] in self.macs):
+            if d.get("mac", None) and (d['mac'] in self.macs):
+                #self.log("d: %s" %(d))
                 # ONly change the mac and ttl on the same host.
                 self.ofctl_add_flow(bridge_name,
                     "priority=1000,ip,nw_dst=%s,action=mod_dl_src:%s,"
@@ -581,16 +621,18 @@ class OvsHost(linux.LinuxDevice):
         # delete all the bridges in the OVS
         bridges = args and args or self.cmd("ovs-vsctl list-br")
         for b in StringIO(bridges).readlines():
-            ports = self.cmd("ovs-vsctl list-ports %s" %(b.strip()))
+            b = b.strip()
+            ports = self.cmd("ovs-vsctl list-ports %s" %(b))
             for p in StringIO(ports).readlines():
                 # delete all the devices(container/vm/physical) connectting to.
-                external_ids = self.ovs_get_interface(p.strip(), "external_ids")
+                p = p.strip()
+                external_ids = self.ovs_get_interface(p, "external_ids")
                 external_ids = external_ids.strip("{} \t\r\n")
 
-                #self.log("bridge: %s, port: %s" %(b.strip(), p.strip()))
-                if (p.strip() in self.nics):
+                #self.log("debug, bridge: %s, port: %s" %(b, p))
+                if (p in self.nics):
                     # Restore the "phisical" interface
-                    self.no_takeover_interface(b.strip(), p)
+                    self.no_takeover_interface(b, p, p)
                 elif ("=" in external_ids):
                     # Parse external_ids: {container_id=con13, container_iface=eth1}
                     i = {}
@@ -600,7 +642,7 @@ class OvsHost(linux.LinuxDevice):
                     if (i.get("container_id", None)):
                         self.cmd('docker stop %s' %(i["container_id"]))
                         self.cmd('docker rm %s' %(i["container_id"]))
-            self.cmd("ovs-vsctl del-br %s" %(b.strip()))
+            self.cmd("ovs-vsctl del-br %s" %(b))
         self.cmd('ovs-vsctl show')
         #self.cmd('ovs-ctl stop')
 
@@ -699,6 +741,59 @@ class OvsHost(linux.LinuxDevice):
                 self.log("FAIL: %s ping %s, %s -> %s!" %(src["name"], dst["name"],
                     srcip.ip, dstip.ip), bg_color = "red")
 
+    def vm_init2 (self, vm, script = None, gateway = None, remote = None):
+        # configure route and execute the script if given
+        u = urlparse.urlparse(vm["url"])
+        c = "ip route add %s via %s dev %s" \
+                %(ipaddress.ip_interface(remote["ip"]).network,
+                    ipaddress.ip_interface(gateway["ip"]).ip, vm["port"][1]["name"])
+        if script:
+            self.cmd("sshpass -p %s scp %s %s@%s:~/" %(u.password, script,
+                u.username, u.hostname))
+            c += "; sh ~/%s" %(script)
+        self.cmd('sshpass -p %s ssh %s@%s "%s"' %(u.password, u.username, u.hostname, c))
+
+    def vm_init (self, *args, gateways = [], script = None):
+        # Given some devivces and gates, configure route automatically
+
+        devices = copy.deepcopy(args)
+        #subnets = (ipaddress.ip_interface(gw["ip"]).network for gw in gateways)
+        for vm in devices:
+            # Find via for every devices
+            vm["ipaddress"] = ipaddress.ip_interface(vm["port"][1]["ip"]).ip
+            for gw in gateways:
+                #self.log("gw: %s, %s!" %(ipaddress.ip_interface(gw["ip"]).ip,
+                #    ipaddress.ip_interface(gw["ip"]).network))
+                if vm["ipaddress"] in ipaddress.ip_interface(gw["ip"]).network:
+                    vm["via"] = ipaddress.ip_interface(gw["ip"]).ip
+
+        for vm in devices:
+            # Configure the route
+            u = urlparse.urlparse(vm["url"])
+            if script:
+                self.cmd("sshpass -p %s scp %s %s@%s:~/" %(u.password, script,
+                    u.username, u.hostname))
+            c = script and ("sh ~/%s; " %(script)) or ""
+
+            for gw in gateways:
+                gw_ipaddress = ipaddress.ip_interface(gw["ip"])
+                #self.log("gw: %s, %s!" %(gw_ipaddress.ip, gw_ipaddress.network))
+                if vm["ipaddress"] not in gw_ipaddress.network:
+                    c += "ip route add %s via %s dev %s" \
+                            %(gw_ipaddress.network, vm["via"],
+                                    vm["port"][1]["name"])
+                    #self.log("%s, route: c: %s" %(vm["name"], c))
+
+            self.cmd('sshpass -p %s ssh %s@%s "%s"' %(u.password,
+                u.username, u.hostname, c))
+
+    def vm_uninit (self, *args):
+        # configure route and execute the script if given
+        for vm in args:
+            u = urlparse.urlparse(vm["url"])
+            self.cmd('sshpass -p %s ssh %s@%s shutdown now' %(u.password,
+                u.username, u.hostname))
+
     def _util_dictionary_parse (self, params):
         '''
         Add some flows to ofproto
